//******************************************************************************
//GoldMap
//by BlueSonic (aka Jigglypuff)
//******************************************************************************
unit MapProperties;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Spin, Map, EventBuilder;

type
  TMapPropertiesForm = class(TForm)
    GroupBox1: TGroupBox;
    Button1: TButton;
    Button2: TButton;
    Label1: TLabel;
    Label2: TLabel;
    AreaCombo: TComboBox;
    Label3: TLabel;
    Label4: TLabel;
    SecondaryHdrEdit: TEdit;
    GroupBox2: TGroupBox;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    BorderCombo: TComboBox;
    HeightSpin: TSpinEdit;
    MapAddrEdit: TEdit;
    WidthSpin: TSpinEdit;
    TilesetCombo: TComboBox;
    MusicCombo: TComboBox;
    Label9: TLabel;
    TypeCombo: TComboBox;
    Button3: TButton;
    EventAddrEdit: TEdit;
    Button4: TButton;
    Label10: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure BorderComboDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure TilesetComboChange(Sender: TObject);
    procedure TilesetComboDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
  public
    MapHdr1Addr: Integer;
    MapHdr2Addr: Integer;
    MapHdr1: TMapHeader1;
    MapHdr2: TMapHeader2;
  end;

var
  MapPropertiesForm: TMapPropertiesForm;

implementation

uses Main,Tileset, DateUtils, ReconstructMap;

{$R *.dfm}

procedure TMapPropertiesForm.FormCreate(Sender: TObject);
var
I: Integer;
begin
AreaCombo.Items := AreaNames;
for I := 0 to Blocks -1 do
BorderCombo.Items.Add(IntToHex(I,2));
for I := 0 to 28 do
TilesetCombo.Items.Add(IntToHex(I,2));
MapHdr1Addr := GetMapHeader1Addr(LoadedMapBank,LoadedMapNum);
MapHdr1 := GetMapHeader1(MapHdr1Addr);
MapHdr2Addr := GBPtrToFilePos(MapHdr1.Header2Bank,MapHdr1.Header2Addr);
MapHdr2 := GetMapHeader2(MapHdr1);
//load values to form
TilesetCombo.ItemIndex := MapHdr1.Tileset;
TypeCombo.ItemIndex := MapHdr1.MapType - 1;
AreaCombo.ItemIndex := MapHdr1.Area;
MusicCombo.ItemIndex  := MapHdr1.Music;
SecondaryHdrEdit.Text := '$'+IntToHex(MapHdr2Addr,6) + ' (' + IntToHex(MapHdr1.Header2Bank,2) + ':' + IntToHex(MapHdr1.Header2Addr,4) + ')';
BorderCombo.ItemIndex := MapHdr2.BorderBlock;
WidthSpin.Value       := MapHdr2.XSize;
HeightSpin.Value      := MapHdr2.YSize;
MapAddrEdit.Text      := '$'+IntToHex(GBPtrToFilePos(MapHdr2.MapDataAddr),6) + ' (' + IntToHex(MapHdr2.MapDataAddr.Bank,2) + ':' + IntToHex(MapHdr2.MapDataAddr.Addr,4) + ')';
EventAddrEdit.Text    := '$'+IntToHex(GBPtrToFilePos(MapHdr2.TextAndEventBank,MapHdr2.EventAddress),6) + ' (' + IntToHex(MapHdr2.TextAndEventBank,2) + ':' + IntToHex(MapHdr2.EventAddress,4) + ')';
end;

procedure TMapPropertiesForm.BorderComboDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
begin
with TComboBox(Control).Canvas do
 begin
 FillRect(Rect);
 TextOut(Rect.Left,Rect.Top,'Block ' + IntToHex(Index,2) + ':');
 Draw((Rect.Right - Rect.Left) div 2 - 32,Rect.Top+13,BlockPics[Index]);
 end;
end;

procedure TMapPropertiesForm.TilesetComboChange(Sender: TObject);
var
I: Integer;
begin
MainForm.LoadTileset(TilesetCombo.ItemIndex);
BorderCombo.Items.Clear;
for I := 0 to Blocks - 1 do
BorderCombo.Items.Add(IntToHex(I,2));
BorderCombo.ItemIndex := 1;
end;

procedure TMapPropertiesForm.TilesetComboDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
Temp: TBitmap;
begin
Temp := TBitmap.Create;
Temp.Width  := 128;
Temp.Height := 48;
Temp.PixelFormat := pf24bit;
ReadTilesetGraphics(GetTilesetHeader(Index),Temp);
with TComboBox(Control).Canvas do
 begin
 FillRect(Rect);
 TextOut(Rect.Left,Rect.Top,'Tileset ' + IntToStr(Index) + ':');
 Draw((Rect.Right - Rect.Left) div 2 - 64,Rect.Top+13,Temp);
 end;
Temp.Free;
end;

procedure TMapPropertiesForm.Button1Click(Sender: TObject);
begin
MapHdr1.Tileset := TilesetCombo.ItemIndex;
MapHdr1.MapType := TypeCombo.ItemIndex + 1;
MapHdr1.Area := AreaCombo.ItemIndex;
MapHdr1.Music := MusicCombo.ItemIndex;
MapHdr2.BorderBlock := BorderCombo.ItemIndex;
MapHdr2.YSize := HeightSpin.Value;
MapHdr2.XSize := WidthSpin.Value;
Rom.Position := MapHdr1Addr;
Rom.Write(MapHdr1,SizeOf(TMapHeader1));
Rom.Position := MapHdr2Addr;
Rom.Write(MapHdr2,SizeOf(TMapHeader2));
Close;
end;

procedure TMapPropertiesForm.Button2Click(Sender: TObject);
begin
Close;
end;

procedure TMapPropertiesForm.Button3Click(Sender: TObject);
begin
with TReconstructMapForm.Create(nil) do
 begin
 ShowModal;
 Free;
 end;
MapHdr2 := GetMapHeader2(MapHdr1);
WidthSpin.Value       := MapHdr2.XSize;
HeightSpin.Value      := MapHdr2.YSize;
MapAddrEdit.Text      := '$'+IntToHex(GBPtrToFilePos(MapHdr2.MapDataAddr),6) + ' (' + IntToHex(MapHdr2.MapDataAddr.Bank,2) + ':' + IntToHex(MapHdr2.MapDataAddr.Addr,4) + ')';
end;

procedure TMapPropertiesForm.Button4Click(Sender: TObject);
begin
with TEventBuilderForm.Create(nil) do
 begin
 ShowModal;
 Free;
 end;
MapHdr2 := GetMapHeader2(MapHdr1);
EventAddrEdit.Text := '$'+IntToHex(GBPtrToFilePos(MapHdr2.TextAndEventBank,MapHdr2.EventAddress),6) + ' (' + IntToHex(MapHdr2.TextAndEventBank,2) + ':' + IntToHex(MapHdr2.EventAddress,4) + ')';
end;

end.
