//******************************************************************************
//GoldMap
//by BlueSonic (aka Jigglypuff)
//******************************************************************************
unit Tileset;

interface

uses Map, Classes, Dialogs, SysUtils, Graphics;

type

//tileset header structure in rom, header array starts at $156BE
TTilesetHeader = packed record
  GraphicsAddr:  TGBPtr;
  MapBlockAddr:  TGBPtr;
  CollisionAddr: TGBPtr;
  AnimationAddr: Word;
  Unused: array [0..1] of Byte;
  PaletteAddr: Word;
  end;

//color palette for tile in rom
TPaletteData = array [0..3] of Word;

var
Palette: array [0..3] of TColor = (clWhite,clLtGray,clDkGray,clBlack);
PaletteIndex: Byte = 1;//default: Day (0: morning, 1: day, 2: night, 3: dark (caves), 4: indoors)

const Zero: Byte = 0;

function GetTilesetHeader(TilesetNumber: Byte): TTilesetHeader;
procedure ReadTilesetGraphics(TilesetHeader: TTilesetHeader; Bmp: TBitmap);

implementation

uses Main;

function GetTilesetHeader(TilesetNumber: Byte): TTilesetHeader;
begin
Rom.Position := $156BE + ((TilesetNumber) * SizeOf(TTilesetHeader));
Rom.Read(Result,SizeOf(TTilesetHeader));
end;

function DecompressGraphic(InputData: TMemoryStream): TMemoryStream;

 procedure WriteReverseBits(Stream: TStream; Data: Byte);
 var
 I: Integer;
 B: Byte;
 begin
 B := 0;
 for I := 0 to 7 do
 B := B or (((Data shr I) and 1) shl (7 - I));
 Stream.Write(B,1);
 end;

var
OutputData: TMemoryStream;
IsDecompressing: boolean;
Tubina,Opcode,Tu,Tu2: Byte;
KLen: Integer;
Low5: Integer;
I: Integer;
label
DecompressionRoutine;
begin
OutputData := TMemoryStream.Create;
OutputData.SetSize($700);
IsDecompressing := True;
 repeat
 InputData.Read(Tubina,1);
 Opcode := Tubina and $E0;
 Low5   := Tubina and $1F;
 Tu   := 0;
 Tu2  := 0;
 if Tubina = $FF then Break;
  try
  DecompressionRoutine:
   case Opcode of
   $00: for I := 0 to Low5 do OutputData.CopyFrom(InputData,1);
   $20:
    begin
    InputData.Read(Tu,1);
    for I := 0 to Low5 do
    OutputData.Write(Tu,1);
    end;
   $40:
    begin
    InputData.Read(Tu ,1);
    InputData.Read(Tu2,1);
    I := 0;
     repeat
     OutputData.Write(Tu ,1);
     I := I + 1;
     if I > Low5 then
     Break;
     OutputData.Write(Tu2,1);
     I := I + 1;
     until I > low5;
    end;
   $60: for I := 0 to Low5 do OutputData.Write(Zero,1);
   $80:
    begin
    InputData.Read(Tu,1);
    if Tu and $80 = 0 then
     begin
     InputData.Read(Tu2,1);
     for I := 0 to Low5 do
     OutputData.Write(pByte(Integer(OutputData.Memory) + ((Tu2 + 1) + (Tu * $100) + I)-1)^,1);
     end
    else
     begin
     Tu2 := Tu and $7F;
     for I := 0 to Low5 do
     OutputData.Write(pByte((Integer(OutputData.Memory) + (OutputData.Position - Tu2) -1))^,1);
     end;
    end;
   $A0:
    begin
    InputData.Read(Tu,1);
    if Tu and $80 = 0 then
     begin
     InputData.Read(Tu2,1);
     for I := 0 to Low5 do
     WriteReverseBits(OutputData,pByte(Integer(OutputData.Memory) + (Tu2 + I + (Tu * $100)))^);
     end
    else
     begin
     Tu2  := Tu and $7F;
     Klen := OutputData.Position;
     for I := 0 to Low5 do
     WriteReverseBits(OutputData,pByte(Integer(OutputData.Memory) + (Klen - Tu2 + I) - 1)^);
     end;
    end;
   $C0:
    begin
    InputData.Read(Tu,1);
    if Tu and $80 = 0 then
     begin
     InputData.Read(Tu2,1);
     for I := 0 to Low5 do
     OutputData.Write(pByte(Integer(OutputData.Memory) + ((Tu2 + 1) + (Tu * $100) - I) - 1)^,1);
     end
    else
     begin
     Tu2  := Tu and $7F;
     Klen := OutputData.Position;
     for I := 0 to Low5 do
     WriteReverseBits(OutputData,pByte(Integer(OutputData.Memory) + ((KLen - Tu2 - I) - 1))^);
     end;
    end;
   $E0:
    begin
    Tubina := (Tubina and $1C) * 8;
    Opcode := (Tubina and $E0);
    InputData.Read(Tu,1);
    Low5   := (Low5 and $3) * $100 + Tu;
    goto DecompressionRoutine;
    end;
   else
    begin
    MessageDlg('Unknown Control Code: ' + IntToHex(Opcode,2),mtWarning,[mbOK],0);
    IsDecompressing := False;
    end;
   end;//case
  except on E: Exception do
   begin
   MessageDlg('Internal error occured while decompressing tileset graphics.',mtError,[mbOK],0);
   IsDecompressing := False;
   end;//except
  end;//try
 until not IsDecompressing;
OutputData.position := 0;
Result := OutputData;
end;


function WordToColor(Value: Word): TColor;
var
R,G,B: Byte;
begin
R := Round((((Value shr 00) and 31) / 31) * 255);
G := Round((((Value shr 05) and 31) / 31) * 255);
B := Round((((Value shr 10) and 31) / 31) * 255);
Result := R or (G shl 8) or (B shl 16);
end;

procedure ReadPalette(Addr: Integer; Pal: array of TColor);
var
I: Integer;
PaletteData: TPaletteData;
begin
Rom.Position := Addr;
Rom.Read(PaletteData,SizeOf(PaletteData));
for I := 0 to 3 do
 begin
  if (PaletteData[I] = $7C81) then
 PaletteData[I] := $7D8D;
 if (PaletteData[I] = $7FFF) then
 PaletteData[I] := $7EF7;
 Palette[I] := WordToColor(PaletteData[I]);
 end;
end;

//reads tile from Stream into Bmp and uses colors from Pal
procedure ReadTile(Stream: TStream; Bmp: TBitmap; Pal: array of TColor);
var
X,Y: Integer;
Data1,Data2: Byte;
begin
for Y := 0 to 7 do
 begin
 Stream.Read(Data1,1);
 Stream.Read(Data2,1);
 for X := 0 to 7 do
 Bmp.Canvas.Pixels[X,Y] := Pal[((Data1 shr (7-X)) and 1) or (((Data2 shr (7-X)) and 1) shl 1)];
 end;
end;

//reads tileset at specific address into Bmp
procedure ReadTilesetGraphics(TilesetHeader: TTilesetHeader; Bmp: TBitmap);
var
Tile: TBitmap;
TileCnt: Byte;
I: Integer;
AttrAddr: Integer;
TileAttr: Byte;
Compressed,Data: TMemoryStream;
begin
Tile := TBitmap.Create;
Tile.PixelFormat := pf24bit;
Tile.Width  := 8;
Tile.Height := 8;
Rom.Position := GBPtrToFilePos(TilesetHeader.GraphicsAddr);
Compressed := TMemoryStream.Create;
Compressed.CopyFrom(Rom,$700);
Compressed.Position := 0;
Data := DecompressGraphic(Compressed);
TileCnt := 0;
AttrAddr := GBPtrToFilePos(2,TilesetHeader.PaletteAddr);
for I := 1 to 16 * 6 do
 begin
 //read palette attribute
 Rom.Position := AttrAddr + (TileCnt div 2);
 Rom.Read(TileAttr,1);
 if (TileCnt and 1) = 1 then
 TileAttr := (TileAttr shr 4) and $F
 else
 TileAttr := TileAttr and $F;
 //read palette
 ReadPalette($B75E + (TileAttr * SizeOf(TPaletteData)) + (8 * SizeOf(TPaletteData) * PaletteIndex),Palette);
 //restore tileset pos
 ReadTile(Data,Tile,Palette);
 Bmp.Canvas.Draw((TileCnt mod 16) * 8,(TileCnt div 16) * 8,Tile);
 Inc(TileCnt);
 end;
end;

end.
